/*
	Dialog Structure
	
	Author: Simon Feltman
	Company: REZN8 Productions
	Date: 02.04.2005
	Requires: rezn8mxs.dlx version:1.28 (REZN8 maxscript extension)

	This structure allows wrapping of standard rollouts with usage to create
	modal and non-modal dialogs via the 'createDialog' MAXScript function.
	Automatic support for docking is added when the 'dockable' property is set to true.
	
	This is a genaral purpose wrapper that automatically saves and loads dialog
	state (position, size, docking, and min/maximized info).
	
	With minimal hooks placed in your rollout you get all the features described above.
	Most of the construction parameters are the same as 'createDialog' uses.
	See 'createDialog' documentation for more information on construction parameters.
	
	Construction:
	
	Dialog
		theRollout:<rollout>		-- The rollout this structure will wrap.
		pos:<Point2>				-- Position of the dialog. default:center or 3dsmax
		width:<Integer>				-- Width of the dialog. default: dialog width
		height:<Integer>			-- Height of the dialog. default: dialog height
		placement:<#minimized|#maximized|#normal>	-- This gets loaded and saved to the ini.
		bgcolor:<Color>				-- Dialog background color. default: standard background color.
		fgcolor:<Color>				-- Dialog text color. default: standard text color
		bitmap:<Bitmap>				-- Background bitmap for the dialog.
		bmpstyle:<Name>				-- default: #bmp_center
		menu:<RCMenu>				-- Menu definition to use as a dialog menu.
		style:<StyleArray>			-- default: #( #style_titlebar, #style_border, #style_sysmenu )
		escapeEnable:<Boolean>		-- Allow escape key to stop MAXScript execution: default: true
		lockHeight:<Boolean>		-- Lock the dialog height. default: false
		lockWidth:<Boolean>			-- Lock the dialog width. default: false
		clipChildren:<Boolean>		-- Clip dialog controls from the redraw area. default: false
		
		-- Dockable dialog bar params. See cui.RegisterDialogBar for more information.
		dockable:<Boolean>			-- Allow this dialog to be docked. default: false
		dockState:<Name>			-- The state the dockable dialog will be opened.
		dockStyle:<StyleArray>		-- Array of docking styles. default: #(#cui_dock_all, #cui_floatable, #cui_handles)
		dockMinSize:<Point2>		-- Minimum size the docking dialog can be. default: dialog size
		dockMaxSize:<Point2>		-- Maximum size the docking dialog can be. default: dialog size
		
		configFile:<Filename>		-- Full path name to the INI file in which settings are saved and loaded.
	
	Usage: A simple example
		The following example creates a simple resizing dialog that saves and loads
		the position and size each time it's loaded. It also returns the currently
		selected item in the list when closed.
	
	>>
		rollout TestRoll "Dialog Test" width:160 height:280
		(
			listBox theList "" pos:[8,8] width:144 height:20
			
			local theDlg = undefined
			
			on TestRoll open do
			(
				-- Get the struct that is wrapping this rollout.
				theDlg = getStructContext()		
				
				-- Get the params passed into the Dialog wrapper
				theList.items = theDlg.params
				
				-- Load dialog states
				theDlg.OnOpen()
			)
			on TestRoll close do
			(
				-- Set the result of this dialog
				theDlg.result = theList.selected
				
				-- Save dialog states
				theDlg.OnClose()
			)
		)

		dlg = Dialog theRollout:TestRoll style:#(#style_titlebar,#style_resizing,#style_sysmenu)
		dlg.DoModal params:#("Item one", "Item 2", "3")
	<<
*/


struct Dialog
(
	-- The structure being constructed
	self = getStructContext(),
	
	-- Rollout definition for this dialog
	theRollout = undefined,
	
	
	-- Helper for getting a MXS color value from colorman.
	fn GetColorValue colorKey =
	(
		local v = colorman.getcolor colorKey
		return (color ( v.z*255) (v.y*255) (v.x*255) )
	),
	
	-- Convert a MAXScript color into a Win32 color
	function ColorToCOLORREF clr =
	(
		local colorref = 0
		colorref += bit.shift (clr.b as integer) 16
		colorref += bit.shift (clr.g as integer) 8
		colorref += (clr.r as integer)
		return colorref
	),
	
	-- Convert MAXScript color into a OLE color (ActiveX controls have R and B flipped)
	function ColorToOLEColor clr =
	(
		return (color clr.b clr.g clr.r)
	),
	
	-- Load an Icon from a toolbar imagelist for use with activeX controls
	-- This function was ripped from the MAXScript tutorials but it's so useful
	-- I thought I'd throw it in here.
	function LoadIcon thePath number width:undefined height:undefined =
	(
		local tempBmp = openBitmap thePath
		if tempBmp == undefined then
			return undefined
		
		-- Attempt to autodetect the width and height
		if width == undefined or height == undefined then
		(
			-- Default to width x height icons
			width = height = tempBmp.height
			
			-- For an image height of 15 use 15x16 icons
			if height == 15 then
				width = 16
		)
		
		local iconBmp = bitmap width height
		for v = 0 to (height-1) do
			setPixels iconBmp [0,v] (getPixels tempBmp [(number-1)*width, v] width)
		
		iconBmp.filename = GetDir #image + "/_temp.bmp"
		save iconBmp
		close iconBmp 
		close tempBmp
		
		return LoadPicture( GetDir #image + "/_temp.bmp")
	),
	
	-- Hit test an ActiveX tree control with the mouse position.
	function HitTestActiveXTree treeControl cursorPos:undefined = 
	(
		if cursorPos == undefined then
			cursorPos = getCursorPos treeControl
		local p = pixelsToTwips (cursorPos)
		return (treeControl.HitTest p.x p.y)
	),
	
	
	-- These are the default CreateDialog params and can be overridden
	-- either during Dialog construction or before the Create or DoModal function is called
	
	-- The following properties get automatically loaded and saved.
	pos = undefined, -- unless overriden, the first time through this will center to 3dsmax main window
	width = theRollout.width,
	height = theRollout.height,
	placement = #normal,
	
	modal = false,
	bgcolor = GetColorValue #background,
	fgcolor = GetColorValue #text,
	bitmap = undefined,
	bmpstyle = #bmp_center,
	menu = undefined,
	style = #( #style_titlebar, #style_border, #style_sysmenu ),
	escapeEnable = true,
	lockHeight = false,
	lockWidth = false,
	clipChildren = false,
	
	-- Dockable dialog bar params
	dockable = false,
	dockState = #cui_floatable, -- saved and restored from ini
	dockStyle = #(#cui_dock_all, #cui_floatable, #cui_handles),
	dockMinSize = [width,height],
	dockMaxSize = [width,height],

	-- Dialog state saving params
	configFile = (getDir #plugcfg) + "\\MAXScriptDialogs.ini",
	
	-- General parameters you can pass into your rollout.
	--  access from within the dialog '(getStructContext()).params'
	params = undefined,
	
	-- Result
	--  set this in the close handler using '(getStructContext()).result = <value>'
	result = undefined,
	
	
	-- An easy value loading function for the INI file. Use for things like
	-- splitter positioning in your rollouts on open handler
	fn LoadSetting key default:undefined =
	(
--		if configFile == undefined then
--			return default
		local val = getINISetting configFile theRollout.name key
		if val.count > 0 then
			return val
		return default
	),
	
	-- Simplified saving of INI values, use in your rollouts on close handler
	-- for things like splitter positioning.
	fn SaveSetting key val =
	(
--		if configFile != undefined then
			setINISetting configFile theRollout.name key (val as String)
	),
	
	-- Attempts to load the state of this dialog (position, width, height, placement, docked)
	fn LoadState =
	(
		val = LoadSetting "Width"
		if val != undefined then
			width = (execute val)
		
		val = LoadSetting "Height"
		if val != undefined then
			height = (execute val)
		
		-- get position and size from ini file
		local val = LoadSetting "Position"
		if val != undefined then
			pos = (execute val)
		else if pos == undefined then
		(
			-- Center dialog to 3dsmax window
			local appsize = getMAXWindowSize()
			pos = (getMAXWindowPos()) + [(appsize.x-width)/2, (appsize.y-height)/2]
		)
		
		val = LoadSetting "Placement"
		if val != undefined then
			placement = val as Name
		
		val = LoadSetting "DockState"
		if val != undefined then
			dockState = val as Name
	),
	
	-- Attempts to save the state of this dialog (position, width, height, placement, docked)
	fn SaveState =
	(
		if theRollout.placement != undefined then
		(
			-- save out the dialogs placment
			SaveSetting "Placement" theRollout.placement
			
			-- make sure to restore to normal before saving position and size
			theRollout.placement = #normal
		)
			
		-- save position and size to ini file
		SaveSetting "Position" (getDialogPos theRollout)
		local size = dialogs.getSize theRollout
		SaveSetting "Width" size.x
		SaveSetting "Height" size.y
	),
	
	-- Call this in your rollouts open handler
	-- These are all things that need to happen post-dialog creation.
	fn OnOpen =
	(
		if clipChildren then
			dialogs.setClipChildren theRollout true
			
		-- Force the resized event handler to be called. 
		try ( theRollout.resized [width,height] ) catch ( )
		
		-- Restore the placement (minimized, maximized, normal)
		theRollout.placement = placement

		-- Register the dialog as a dialogbar
		if dockable then
		(
			cui.RegisterDialogBar theRollout style:dockStyle minSize:dockMinSize maxSize:dockMaxSize
			
			-- Dock according to dockState
			cui.DockDialogBar theRollout dockState
		)
	),
	
	-- Call this in your rollouts close handler
	fn OnClose =
	(
		-- If it's dockable then save out the dock state and unregister
		if dockable then
		(
			-- NOTE: If OnClose is called inside the dialogs 'on close' handler
			-- theRollout.dialogBar will alway be false. This is because at that
			-- point the dialog has already been un-registered as a dialog bar
			-- by the system.
			if theRollout.dialogBar then
			(
				local state = cui.getDockState theRollout
				SaveSetting "DockState" state
				cui.UnRegisterDialogBar theRollout
			)
			else
				SaveSetting "DockState" #cui_floatable
			
			-- Compensate 4 pixels when a dialog is unregistered as dialogbar
			local pos = getDialogPos theRollout
			setDialogPos theRollout [pos.x, pos.y-4]
		)

		SaveState()
	),
	
	-- Standard dialog creation
	fn Create params:undefined =
	(
		if params != undefined then
			self.params = params

		-- Attempt to load the configuration file before dialog creation
		LoadState()
		
		if menu != undefined then
		(
			createDialog theRollout \
				pos:pos \
				width:width \
				height:height \
				modal:modal \
				menu:menu \
				bgcolor:bgcolor \
				fgcolor:fgcolor \
				bitmap:bitmap \
				bmpstyle:bmpstyle \
				style:style \
				escapeEnable:escapeEnable \
				lockHeight:lockHeight \
				lockWidth:lockWidth
		)
		else
		(
			createDialog theRollout \
				pos:pos \
				width:width \
				height:height \
				modal:modal \
				bgcolor:bgcolor \
				fgcolor:fgcolor \
				bitmap:bitmap \
				bmpstyle:bmpstyle \
				style:style \
				escapeEnable:escapeEnable \
				lockHeight:lockHeight \
				lockWidth:lockWidth
		)
	),
	
	-- Modal creation of this dialog, always modal and always returns result.
	fn DoModal params:undefined =
	(
		self.modal = true
		Create params:params
		return self.result
	),
	
	fn Close result:undefined =
	(
		OnClose()
		destroyDialog theRollout
		if result != undefined then
			self.result = result
	)
)

